import threading
import time
import os
import subprocess
import json
import math
import shutil 
import copy
import psutil


class OnedriveManager:
	
	def __init__(self):

		self.user=os.environ["USER"]
		self.userFolder="/home/%s/OneDrive"%(self.user)
		self.configTemplate="/usr/share/lliurex-onedrive/llx-data/config"
		self.authUrl="https://login.microsoftonline.com/common/oauth2/v2.0/authorize?client_id=d50ca740-c83f-4d1b-b616-12c519384f0c&scope=Files.ReadWrite%20Files.ReadWrite.all%20Sites.Read.All%20Sites.ReadWrite.All%20offline_access&response_type=code&redirect_uri=https://login.microsoftonline.com/common/oauth2/nativeclient"
		self.userTokenPath="/home/%s/.onedriveAuth/"%(self.user)
		self.testPath=os.path.join(self.userTokenPath,"test_onedrive.txt")
		self.onedriveUrl="https://login.microsoftonline.com/common/oauth2/v2.0/authorize?client_id=d50ca740-c83f-4d1b-b616-12c519384f0c&scope=Files.ReadWrite%20Files.ReadWrite.all%20Sites.Read.All%20Sites.ReadWrite.All%20offline_access&response_type=code&redirect_uri=https://login.microsoftonline.com/common/oauth2/nativeclient"
		self.authToken=""
		self.bandWidth=[{"name":"128 KB/s","value":"131072"},{"name":"256 KB/s","value":"262144"},{"name":"512 KB/s","value":"524288"},{"name":"1 MB/s","value":"1048576"},{"name":"10 MB/s","value":"10485760"},{"name":"20 MB/s","value":"20971520"},{"name":"30 MB/s","value":"31457280"},{"name":"50 MB/s","value":"52428800"},{"name":"100 MB/s","value":"104857600"}]
		self.bandWidthNames=[]
		for item in self.bandWidth:
			self.bandWidthNames.append(item["name"])
		self.internalOnedriveFolder="/home/%s/.config/onedrive"%(self.user)
		self.configFile=os.path.join(self.internalOnedriveFolder,"config")
		self.systemdFile="/home/%s/.config/systemd/user/onedrive.service"%(self.user)
		self.autoStartEnabled=True
		self.rateLimit=2
		self.monitorInterval=1
		self.currentConfig=[self.autoStartEnabled,self.monitorInterval,self.rateLimit]
		self.folderStruct=[]
		self.folderStructBack=[]
		self.foldersSelected=[]
		self.foldersUnSelected=[]
		self.filterFile=os.path.join(self.internalOnedriveFolder,"sync_list")
		self.filterFileHash=os.path.join(self.internalOnedriveFolder,".sync_list.hash")
		self.includeFolders=[]
		self.excludeFolders=[]
		self.syncAll=True
		self.currentSyncConfig=[self.syncAll,self.foldersSelected,self.foldersUnSelected]
		self.clearCache()
		self.emptyToken=os.path.join(self.internalOnedriveFolder,".emptyToken")
		self.localFolderEmptyToken=os.path.join(self.internalOnedriveFolder,".localFolderEmptyToken")
		self.localFolderRemovedToken=os.path.join(self.internalOnedriveFolder,".localFolderRemovedToken")
		self.lockAutoStartToken=os.path.join(self.internalOnedriveFolder,".lockAutoStartToken")
		self.envConfFiles=[".config.backup",".config.hash","items.sqlite3","items.sqlite3-shm","items.sqlite3-wal"]
	
	#def __init__

	def loadConfg(self):

		if self.isConfigured():
			self.readConfigFile()
			if not self.isAutoStartEnabled():
				self.autoStartEnabled=False
				self.currentConfig[0]=False

			if self.existsFilterFile():
				self.syncAll=False
				self.readFilterFile()
				self.currentSyncConfig[0]=self.syncAll
				self.currentSyncConfig[1]=self.foldersSelected
				self.currentSyncConfig[2]=self.foldersUnSelected
		
	#def loadConfg
	
	def isConfigured(self):

		token=os.path.join(self.internalOnedriveFolder,"refresh_token")

		if os.path.exists(token):
			return True
		else:
			return False

	#def isConfigured

	def readConfigFile(self):

		if os.path.exists(self.configFile):
			with open(self.configFile,'r') as fd:
				lines=fd.readlines()
				for line in lines:
					if 'monitor_interval' in line:
						value=line.split("=")[1].split("\n")[0].strip().split('"')[1]
						self.monitorInterval="{:.0f}".format(int(value)/60)
						self.currentConfig[1]=self.monitorInterval

					elif 'rate_limit' in line:
						value=line.split("=")[1].split("\n")[0].strip().split('"')[1]
						for i in range(len(self.bandWidth)):
							if self.bandWidth[i]["value"]==value:
								self.rateLimit=i
								self.currentConfig[2]=self.rateLimit
								break
				fd.close()
		else:
			shutil.copyfile(self.configTemplate,self.configFile)
	
	
	#def readConfigFile

	def createAccount(self):

		if not os.path.exists(self.userTokenPath):
			os.mkdir(self.userTokenPath)

		urlDoc=os.path.join(self.userTokenPath,"urlToken")
		tokenDoc=os.path.join(self.userTokenPath,"keyToken")
		f=open(urlDoc,'w')
		f.write(self.onedriveUrl)
		f.close()
		f=open(tokenDoc,'w')
		f.write(self.authToken)
		f.close()

		cmd="/usr/bin/onedrive --auth-files %s:%s"%(urlDoc,tokenDoc)
		p=subprocess.Popen(cmd,shell=True,stdout=subprocess.PIPE)
		poutput=p.communicate()
		rc=p.returncode
		if rc in [0,1]:
			self.readConfigFile()
			if not self.isAutoStartEnabled():
				self.autoStartEnabled=False
				self.currentConfig[0]=False

			self._manageEmptyToken()
			#ret=self.manageSync(True)
			return True
		else:
			return False

	#def createAccount

	def getInitialDownload(self):

		download=""
		cmd="/usr/bin/onedrive --display-sync-status"
		p=subprocess.Popen(cmd,shell=True,stdout=subprocess.PIPE)
		poutput=p.communicate()[0]
		rc=p.returncode

		if rc==0:
			if type(poutput) is bytes:
				poutput=poutput.decode()

			poutput=poutput.split("\n")

			for line in poutput:
				if 'data to download' in line:
					tmpLine=line.split(":")[1].strip()
					if tmpLine!="":
						download=self._formatInitialDownload(tmpLine)
						break
		
		return download

	#def getInitialDownload

	def _formatInitialDownload(self,value):

		if "KB" in value:
			tmp=value.split(" ")[0]
			tmp=int(tmp)*1024
		elif "MB" in value:
			tmp=value.split(" ")[0]
			tmp=int(tmp)*1024*1024
		elif "GB" in value:
			tmp=value.split(" ")[0]
			tmp=int(tmp)*1024*1024*1024
		else:
			tmp=value.split(" ")[0]
			tmp=int(tmp)

		return self._formatFreeSpace(tmp)
	
	#def _formatInitialDownload

	def getHddFreeSpace(self):

		hdd=psutil.disk_usage('/home')
		hddFreeSpace=self._formatFreeSpace(hdd.free)
		return hddFreeSpace

	#def getHddFreeSpace

	def isAutoStartEnabled(self):

		if os.path.exists(self.systemdFile) and not os.path.exists(self.lockAutoStartToken):
			return False

		return True

	#def isAutostartEnabled

	def isOnedriveRunning(self):

		if os.system("ps -ef | grep 'onedrive --monitor' | grep -v 'grep' 1>/dev/null")==0:
			return True
		else:
			return False

	#def isOnedriveRunning

	def applyChanges(self,value):

		SYSTEMD_ERROR=-10
		WRITE_CONFIG_ERROR=-20
		MULTIPLE_SETTINGS_ERROR=-30

		errorSD=False
		errorMI=False
		errorRL=False

		if value[0]!=self.currentConfig[0]:
			errorSD=self.manageAutostart(value[0])
			if not errorSD:
				self.currentConfig[0]=value[0]
			
		if value[1]!=self.currentConfig[1]:
			errorMI=self.manageMonitorInterval(value[1])
			if not errorMI:
				self.currentConfig[1]=value[1]
			
		if value[2]!=self.currentConfig[2]:
			errorRL=self.manageRateLimit(value[2])
			if not errorRL:
				self.currentConfig[2]=value[2]
			
		
		if errorSD and not errorMI and not errorRL:
			return[True,SYSTEMD_ERROR]

		elif errorSD and (errorMI or errorRL):
			return [True,MULTIPLE_SETTINGS_ERROR]

		elif not errorSD and (errorMI or errorRL):
			return [True,WRITE_CONFIG_ERROR]

		elif not errorSD and errorMI and errorRL:
			return [True,WRITE_CONFIG_ERROR]

		else:
			return[False,'']

	#def applyChanges
	
	def manageAutostart(self,value):

		isOnedriveRunning=self.isOnedriveRunning()

		if value:
			cmd="systemctl --user unmask onedrive.service"
			
		else:
			cmd="systemctl --user mask onedrive.service"

		p=subprocess.Popen(cmd,shell=True,stdout=subprocess.PIPE)
		poutput=p.communicate()
		rc=p.returncode

		if rc !=0:
			return True
		else:
			return False
	
	#def manageAutostart

	def manageMonitorInterval(self,value):

		value=str(int(value)*60)
		return self._writeConfigFile('monitor_interval',value)

	#def manageMonitorInterval

	def manageRateLimit(self,index):

		value=self.bandWidth[index]["value"]
		return self._writeConfigFile('rate_limit',value)

	#def manageRateLimit

	def _writeConfigFile(self,param,value):

		if os.path.exists(self.configFile):
			try:
				with open(self.configFile,'r') as fd:
					lines=fd.readlines()
					fd.close()

				with open(self.configFile,'w') as fd:

					for line in lines:
						if  param in line:
							tmp_line=param+' = '+'"'+value+'"\n'
							fd.write(tmp_line)
						else:
							fd.write(line)

					fd.close()
					return False
			except:
				return True
		else:
			return True

	#def _writeConfigFile

	def manageSync(self,value):

		if value:
			self.manageFileFilter("restore")
			if self.isAutoStartEnabled():
				if os.path.exists(self.lockAutoStartToken):
					ret=self.manageAutostart(True)
				cmd="systemctl --user start onedrive.service"
			else:
				cmd="/usr/bin/onedrive --monitor &"
		else:
			if self._isSystemdActive():
				cmd="systemctl --user stop onedrive.service"
			else:
				cmd="ps -ef | grep 'onedrive --monitor' | grep -v grep | awk '{print $2}' | xargs kill -9"				
		
		try:
			if os.path.exists(self.localFolderEmptyToken):
				self._manageEmptyToken()
			p=subprocess.run(cmd,shell=True,check=True)
			return True
		except subprocess.CalledProcessError as e:
			return False

	#def manageSync

	def _isSystemdActive(self):

		cmd="systemctl --user is-active onedrive.service"
		p=subprocess.Popen(cmd,shell=True,stdout=subprocess.PIPE)
		poutput=p.communicate()[0]
		
		if type(poutput) is bytes:
			poutput=poutput.decode()		
		poutput=poutput.split('\n')[0]

		if poutput=="active":
			return True
		else:
			return False

	#def _isSystemdActive

	def removeAccount(self):

		if self.isOnedriveRunning():
			ret=self.manageSync(False)
			
		if not self.isOnedriveRunning():
			cmd="/usr/bin/onedrive --logout &"
			p=subprocess.run(cmd,shell=True,check=True)
			time.sleep(2)
			if not self.isConfigured():
				self._removeEnvConfigFiles()
				return True
			else:
				return False
		else:
			return False
	
	#def removeAccount

	def getAccountStatus(self):

		MICROSOFT_API_ERROR="-1"
		UNABLE_CONNECT_MICROSOFT_ERROR="-2"
		LOCAL_FILE_SYSTEM_ERROR="-3"
		ZERO_SPACE_AVAILABLE_ERROR="-4"
		QUOTA_RESTRICTED_ERROR="-5"
		DATABASE_ERROR="-6"
		UNAUTHORIZED_ERROR="-7"
		UPLOADING_CANCEL_ERROR="-8"
		SERVICE_UNAVAILABLE="-9"

		ALL_SYNCHRONIZE_MSG="0"
		OUT_OF_SYNC_MSG="2"
		WITH_OUT_CONFIG="1"


		error=False
		code=""
		freespace=""
		if self.isConfigured():
			cmd="/usr/bin/onedrive --display-sync-status --verbose"
			p=subprocess.Popen(cmd,shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
			poutput,perror=p.communicate()

			if type(poutput) is bytes:
				poutput=poutput.decode()

			if type(perror) is bytes:
				perror=perror.decode()

			if len(perror)>0:
				perror=perror.split('\n')
				error=True
				for item in perror:
					if 'OneDrive API returned an error' in item:
						code=MICROSOFT_API_ERROR
					elif 'Cannot connect to' in item:
						code=UNABLE_CONNECT_MICROSOFT_ERROR
						break
					elif 'local file system' in item:
						code=LOCAL_FILE_SYSTEM_ERROR
					elif 'zero space available' in item:
						code=ZERO_SPACE_AVAILABLE_ERROR
						break
					elif 'quota information' in item:
						code=QUOTA_RESTRICTED_ERROR
					elif 'database' in item:
						code=DATABASE_ERROR
					elif 'Unauthorized' in item:
						code=UNAUTHORIZED_ERROR
					elif '416' in item:
						code=UPLOADING_CANCEL_ERROR
						break
					elif 'Unable to query OneDrive' in item:
						code=UNAUTHORIZED_ERROR
						break
					elif '503' in item:
						code=SERVICE_UNAVAILABLE
					elif 'Free Space' in item:
						tmp_freespace=item.split(':')[1].strip()
						freespace=self._formatFreeSpace(tmp_freespace)

			else:
				poutput=poutput.split('\n')
				for item in poutput:
					if 'No pending' in item:
						code=ALL_SYNCHRONIZE_MSG
					elif 'out of sync' in item:
						code=OUT_OF_SYNC_MSG
					elif 'HTTP 403 - Forbidden' in item:
						code=UNAUTHORIZED_ERROR
						error=True
						break
					elif 'Free Space' in item:
						tmp_freespace=item.split(':')[1].strip()
						freespace=self._formatFreeSpace(tmp_freespace)

		else:
			error=True
			code=WITH_OUT_CONFIG

		return [error,code,freespace]

	#def getAccountStatus

	def _formatFreeSpace(self,freespace):

		size_name = ("B", "KB", "MB", "GB","TB")

		size_bytes=float(freespace)
		
		if size_bytes==0:
			return '0 B'

		i = int(math.floor(math.log(size_bytes, 1024)))
		p = math.pow(1024, i)
		s = round(size_bytes/p,2)
		unit=size_name[i]

		return str(s)+" "+unit

	#def _formatFreeSpace

	def repairOnedrive(self):

		running=self.isOnedriveRunning()
		if os.path.exists(self.localFolderRemovedToken):
			self._manageEmptyToken()
		ret=self._syncResync()
		return ret

	#def repairDB

	def _syncResync(self):

		cmd="/usr/bin/onedrive --synchronize --resync"

		p=subprocess.Popen(cmd,shell=True,stdout=subprocess.PIPE)
		ret=p.communicate()
		rc=p.returncode

		if rc!=0:
			return False
		else:
			return True

	#def _syncResync
	
	def testOnedrive(self):

		if not os.path.exists(self.userTokenPath):
			os.mkdir(self.userTokenPath)

		if os.path.exists(self.testPath):
			os.remove(self.testPath)

		cmd="echo SYNC-DISPLAY-STATUS >>%s"%self.testPath
		os.system(cmd)
		cmd="/usr/bin/onedrive --display-sync-status --verbose >>%s 2>&1"%self.testPath
		p=subprocess.call(cmd,shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE)

		cmd="echo TEST SYNCHRONIZE >>%s"%self.testPath
		os.system(cmd)
		cmd="/usr/bin/onedrive --synchronize --dry-run --verbose >>%s 2>&1"%self.testPath
		p=subprocess.call(cmd,shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE)

		return

	#def testOnedrive

	def existsFilterFile(self):

		if os.path.exists(self.filterFile):
			return True
		else:
			if os.path.exists(self.filterFile+".back"):
				self.manageFileFilter("restore")
				return True
			else:
				return False

	#def existsFilterFile
	
	def readFilterFile(self):

		self.includeFolders=[]
		self.excludeFolders=[]

		with open(self.filterFile,'r') as fd:
			lines=fd.readlines()
			fd.close()

			for line in lines:
				tmpLine=line.split("\n")[0]
				if tmpLine.startswith("!"):
					self.excludeFolders.append(tmpLine)
					tmpLine=tmpLine.split("!")[1].split("/*")[0]
					if tmpLine not in self.foldersUnSelected:
						self.foldersUnSelected.append(tmpLine)
				else:
					self.includeFolders.append(tmpLine)
					tmpLine=tmpLine.split("/*")[0]
					if tmpLine not in self.foldersSelected:
						self.foldersSelected.append(tmpLine)

	#def readFilterFile			

	def getFolderStruct(self,localFolder=False):

		if localFolder:
			if os.path.exists(self.userFolder):
				if os.listdir(self.userFolder):
					return self.getLocalFolderStruct()
				else:
					return self.getCloudFolderStruct()
		else:
			return self.getCloudFolderStruct()

	#def getFolderStruct	


	def getCloudFolderStruct(self):
		
		error=False
		if not self.isOnedriveRunning():
			self.manageFileFilter("move")

		cmd='onedrive --synchronize --resync --dry-run --verbose'
		p=subprocess.Popen(cmd,shell=True,stdout=subprocess.PIPE)
		out=p.communicate()[0]
		out=out.decode().split("\n")
		syncOut=copy.deepcopy(out)
		rc=p.returncode

		if not self.isOnedriveRunning():
			self.manageFileFilter("restore")
		
		if rc==0:
			folderResyncStruct=self._processingResyncOut(out)
			folderSyncStruct=self._processingSyncOut(syncOut)

			if len(folderResyncStruct)>0:
				for item in folderSyncStruct:
					match=0
					for element in folderResyncStruct:
						if item["path"]==element["path"]:
							match=0
							break
						else:
							match+=1
					if match>0:
						folderResyncStruct.append(item)
			else:
				folderResyncStruct=folderSyncStruct

			self.folderStruct=sorted(folderResyncStruct,key=lambda d: d['path'])
			self._processingFolderStruct()			
			self.folderStructBack=copy.deepcopy(self.folderStruct)
		else:
			error=True
		
		return [error,self.folderStruct]

	#def getCloudFolderStruct

	def _processingResyncOut(self,out):

		for i in range(len(out)-1,-1,-1):
			if 'local directory' in out[i]:
					pass
			else:
				if 'creating file' in out[i]:
					pass
				else:
					out.pop(i)				

		folderResyncStruct=[]
		for item in out:
			if 'local directory' in item:
				countChildren=0
				tmpList={}
				tmpEntry=item.split(":")[1].strip()
				tmpList["path"]=tmpEntry
				tmpEntry=tmpEntry.split("/")
				tmpList["isChecked"]=True
				tmpList["isExpanded"]=True
				tmpList["hide"]=False
				if len(tmpEntry)==1:
					tmpList["name"]=tmpEntry[0]
					tmpList["type"]="OneDrive"
					tmpList["subtype"]="parent"
					tmpList["level"]=3

				else:
					tmpList["name"]=tmpEntry[-1]
					tmpList["type"]=tmpEntry[-2]
					tmpList["subtype"]="parent"
					tmpList["level"]=len(tmpEntry)*3

				for j in range(0,len(out),1):
					tmpItem2=out[j]
					if 'local directory' in tmpItem2:
						tmpEntry2=out[j].split(":")[1].strip()
						tmpPath=tmpList["path"]+"/"
						if tmpPath in tmpEntry2:
							countChildren+=1

				if countChildren>0:
					tmpList["canExpanded"]=True 
				else:
					tmpList["canExpanded"]=False
				folderResyncStruct.append(tmpList)	

		return folderResyncStruct

	#def _processingResyncOut

	def _processingSyncOut(self,syncOut):		
		
		for i in range(len(syncOut)-1,-1,-1):
			if 'Processing ' in syncOut[i]:
				pass
			else:
				if 'The directory' in syncOut[i]:
					pass
				else:
					if 'The file' in syncOut[i]:
						pass
					else:
						syncOut.pop(i)				

		for i in range(len(syncOut)-1,-1,-1):
			try:
				if 'local state' in syncOut[i]:
					syncOut.pop(i)
				if 'last modified time' in syncOut[i]:
					syncOut.pop(i)	
			except:
				pass
		
		folderSyncStruct=[]
		for i in range(0,len(syncOut)-1,2):
			tmp={}
			tmpItem=syncOut[i]+": "+syncOut[i+1]
			if 'The directory' in tmpItem:
				countChildren=0
				tmpList={}
				tmpEntry=syncOut[i].split("Processing")[1].strip()
				tmpList["path"]=tmpEntry
				tmpEntry=tmpEntry.split("/")
				tmpList["isChecked"]=True
				tmpList["isExpanded"]=True
				tmpList["hide"]=False
				if len(tmpEntry)==1:
					tmpList["name"]=tmpEntry[0]
					tmpList["type"]="OneDrive"
					tmpList["subtype"]="parent"
					tmpList["level"]=3

				else:
					tmpList["name"]=tmpEntry[-1]
					tmpList["type"]=tmpEntry[-2]
					tmpList["subtype"]="parent"
					tmpList["level"]=len(tmpEntry)*3

				
				for j in range(0,len(syncOut)-1,2):
					tmpItem2=syncOut[j]+": "+syncOut[j+1]
					if 'The directory' in tmpItem2:
						tmpEntry2=syncOut[j].split("Processing")[1].strip()
						tmpPath=tmpList["path"]+"/"
						if tmpPath in tmpEntry2:
							countChildren+=1

				if countChildren>0:
					tmpList["canExpanded"]=True 
				else:
					tmpList["canExpanded"]=False
				
				folderSyncStruct.append(tmpList)	
			
		try:
			folderSyncStruct.pop(0)
		except Exception as e:
			pass

		return folderSyncStruct

	#def _processingSyncOut

	def _processingFolderStruct(self):

		if self.existsFilterFile():
			self.readFilterFile()
			for item in self.folderStruct:
				tmp="!"+item["path"]+"/*"
				if tmp in self.excludeFolders:
					item["isChecked"]=False
				else:
					if (item["path"]+"/*") not in self.includeFolders:
						tmp=item["type"]+"/*"
						if tmp in self.includeFolders:
							item["isChecked"]=True
						else:
							tmp=item["path"]+"/*"
							for element in self.includeFolders:
								if element.split("/*")[0] in tmp:
									item["isChecked"]=True
									break
								else:	
									item["isChecked"]=False


	#def _processingFolderStruct

	def getLocalFolderStruct(self):

		try:
			folderLocalStruct=self._processingLocalFolder()
			error=False
		except:
			error=True

		self.folderStruct=sorted(folderLocalStruct,key=lambda d: d['path'])
		self._processingFolderStruct()					
		self.folderStructBack=copy.deepcopy(self.folderStruct)
	
		return [error,self.folderStruct]

	#def getLocalFolderStruct

	def _processingLocalFolder(self):

		folderLocalStruct=[]
		directory=[]
		tmpFolders=[]

		if os.path.exists(self.userFolder):
			for base,dirs,file in os.walk(self.userFolder):
				if base !=self.userFolder:
					directory.append(base)
			
			for item in directory:
				path=os.path.realpath(item)
				tmpFolders.append(path)
			
			
			for item in tmpFolders:
				countChildren=0
				tmpList={}
				tmpEntry=item.split(self.userFolder+"/")[1]
				tmpList["path"]=tmpEntry
				tmpEntry=tmpEntry.split("/")
				tmpList["isChecked"]=True
				tmpList["isExpanded"]=True
				tmpList["hide"]=False
				if len(tmpEntry)==1:
					tmpList["name"]=tmpEntry[0]
					tmpList["type"]="OneDrive"
					tmpList["subtype"]="parent"
					tmpList["level"]=3

				else:
					tmpList["name"]=tmpEntry[-1]
					tmpList["type"]=tmpEntry[-2]
					tmpList["subtype"]="parent"
					tmpList["level"]=len(tmpEntry)*3

				for j in range(0,len(tmpFolders),1):
					tmpItem2=tmpFolders[j]
					tmpEntry2=tmpFolders[j]
					tmpPath=tmpList["path"]+"/"
					if tmpPath in tmpEntry2:
						countChildren+=1

				if countChildren>0:
					tmpList["canExpanded"]=True 
				else:
					tmpList["canExpanded"]=False
				folderLocalStruct.append(tmpList)	
		
		return folderLocalStruct

	#def _processingLocalFolder		

	def applySyncChanges(self,initialSyncConfig,keepFolders):

		syncAll=initialSyncConfig[0]
		foldersSelected=initialSyncConfig[1]
		foldersUnSelected=initialSyncConfig[2]

		if not syncAll:
			self.createFilterFile(foldersSelected,foldersUnSelected)
			if not keepFolders:
				self._manageEmptyToken()
				if os.path.exists(self.userFolder):
					shutil.rmtree(self.userFolder)
			self.readFilterFile()
		else:
			#foldersSelected=[]
			#foldersUnSelected=[]
			if os.path.exists(self.filterFile):
				os.remove(self.filterFile)
			if os.path.exists(self.filterFileHash):
				os.remove(self.filterFileHash)

			#self.folderStruct=[]

		self.currentSyncConfig[0]=syncAll
		self.currentSyncConfig[1]=foldersSelected
		self.currentSyncConfig[2]=foldersUnSelected
		self.folderStructBack=copy.deepcopy(self.folderStruct)

		ret=self._syncResync()
		return ret

	#def applySyncChanges

	def createFilterFile(self,foldersSelected,foldersUnSelected):

		folderSelected=[]
		folderUnSelected=[]
		
		for item in self.folderStruct:
			count=0
			if item["isChecked"]:
				if item["path"] not in foldersUnSelected and item["path"] not in foldersSelected:
					if item["type"]=="OneDrive":
						for element in foldersUnSelected:
							if item["path"] in element:
								break
							else:
								count+=1
						if count>0:
							folderSelected.append(item["path"]+"/*")

		
		for i in range(len(foldersUnSelected)-1,-1,-1):
			for element in foldersSelected:
				if len(self.includeFolders)>0:
					tmp=foldersUnSelected[i]+"/*"
					if tmp in self.includeFolders:
						self.includeFolders.remove(tmp)
				tmpFolder=foldersUnSelected[i]+"/"
				if tmpFolder in element:
					foldersUnSelected.pop(i)

		
		for element in foldersSelected:
			folderSelected.append(element+"/*")

		for element in foldersUnSelected:
			folderUnSelected.append("!"+element+"/*")
					
		
		if not self.existsFilterFile():
			with open(self.filterFile,'w') as fd:
				
				for item in folderUnSelected:
					tmpLine=item+"\n"
					fd.write(tmpLine)

				for item in folderSelected:
					tmpLine=item+"\n"
					fd.write(tmpLine)
				fd.close()
		else:
			for item in folderUnSelected:
				if item not in self.excludeFolders:
					self.excludeFolders.append(item)
			
			for item in folderSelected:
				if item not in self.includeFolders:
					self.includeFolders.append(item)
			
			for i in range(len(self.excludeFolders)-1,-1,-1):
				tmpLine=self.excludeFolders[i].split("!")[1]
				match=0
				if tmpLine in folderSelected:
					self.excludeFolders.pop(i)
				else:
					for item in self.folderStruct:
						tmpItem=item["path"]+"/*"
						if tmpItem==tmpLine:
							match=0
							break
						else:
							match+=1
					if match>0:
						self.excludeFolders.pop(i)
			
			for i in range(len(self.includeFolders)-1,-1,-1):
				tmpLine="!"+self.includeFolders[i]
				match=0
				if tmpLine in folderUnSelected:
					self.includeFolders.pop(i)
				else:
					for item in self.folderStruct:
						tmpItem="!"+item["path"]+"/*"
						if tmpItem==tmpLine:
							match=0
							break
						else:
							match+=1
					if match>0:
						self.includeFolders.pop(i)
			
			for i in range(len(self.excludeFolders)-1,-1,-1):
				for element in self.includeFolders:
					tmp=self.excludeFolders[i].split("!")[1].split("/*")[0]+"/"
					if tmp in element:
						self.excludeFolders.pop(i)

			with open(self.filterFile,'w') as fd:
				
				for element in self.excludeFolders:
					fd.write(element+"\n")
				
				for element in self.includeFolders:
					fd.write(element+"\n")
				
				fd.close()
	
	#def createFilterFile

	def manageFileFilter(self,action):

		if action=="move":
			if os.path.exists(self.filterFile):
				os.rename(self.filterFile,self.filterFile+".back")
			if os.path.exists(self.filterFileHash):
				os.rename(self.filterFileHash,self.filterFileHash+".back")				
		elif action=="restore":
			if os.path.exists(self.filterFile+".back"):
				os.rename(self.filterFile+".back",self.filterFile)
			if os.path.exists(self.filterFileHash+".back"):
				os.rename(self.filterFileHash+".back",self.filterFileHash)

	#def manageFileFilter
		
	def updateCheckFolder(self,path,checked):

		for item in self.folderStruct:
			if item["path"]==path:
				item["isChecked"]=checked

	#def updateCheckFolder

	def cancelSyncChanges(self):

		if self.folderStruct!=self.folderStructBack:
			self.folderStruct=copy.deepcopy(self.folderStructBack)

	#def cancelSyncChanges

	def clearCache(self):

		clear=False
		versionFile="/home/%s/.config/lliurex-onedrive.conf"%self.user
		cachePath1="/home/%s/.cache/lliurex-onedrive"%self.user
		installedVersion=self.getPackageVersion()

		if not os.path.exists(versionFile):
			with open(versionFile,'w') as fd:
				fd.write(installedVersion)
				fd.close()

			clear=True

		else:
			with open(versionFile,'r') as fd:
				fileVersion=fd.readline()
				fd.close()

			if fileVersion!=installedVersion:
				with open(versionFile,'w') as fd:
					fd.write(installedVersion)
					fd.close()
				clear=True
		
		if clear:
			if os.path.exists(cachePath1):
				shutil.rmtree(cachePath1)

	#def clearCache

	def getPackageVersion(self):

		command = "LANG=C LANGUAGE=en apt-cache policy lliurex-onedrive"
		p = subprocess.Popen(command,shell=True,stdout=subprocess.PIPE)
		installed = None
		for line in iter(p.stdout.readline,b""):
			if type(line) is bytes:
				line=line.decode()

			stripedline = line.strip()
			if stripedline.startswith("Installed"):
				installed = stripedline.replace("Installed: ","")

		return installed

	#def getPackageVersion

	def checkLocalFolder(self):

		localFolderEmpty=False;
		localFolderRemoved=False;

		if os.path.exists(self.localFolderEmptyToken):
			localFolderEmpty=True;
		if os.path.exists(self.localFolderRemovedToken):
			localFolderRemoved=True;

		return [localFolderEmpty,localFolderRemoved]

	#def checkLocalFolder

	def _manageEmptyToken(self):

		f=open(self.emptyToken,'w')
		f.close()

	#def _manageEmptyToken

	def checkPreviousLocalFolder(self):

		if os.path.exists(self.userFolder):
			if os.listdir(self.userFolder):
				return True

		return False

	#def checkPreviousLocalFolder

	def _removeEnvConfigFiles(self):

		for item in self.envConfFiles:
			tmpPath=os.path.join(self.internalOnedriveFolder,item)
			if os.path.exists(tmpPath):
				os.remove(tmpPath)

		if os.path.exists(self.filterFile):
			os.remove(self.filterFile)
		if os.path.exists(self.filterFileHash):
			os.remove(self.filterFileHash)

	#def _removeEnvConfigFiles

#class OnedriveManager
